<?php

namespace Drupal\cforge;

use Drupal\mcapi\Entity\Currency;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\node\Entity\Node;
use Drupal\file\Entity\File;
use Drupal\user\Entity\User;
use Drupal\field\Entity\FieldConfig;
use Drupal\Core\Url;
use Drupal\Core\Render\Element;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\ModuleInstallerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Locale\CountryManagerInterface;
use Drupal\Core\Utility\LinkGeneratorInterface;
use Drupal\Core\Extension\ExtensionDiscovery;
use Drupal\Core\Extension\ExtensionList;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Extension\ModuleHandler;


ini_set('max_execution_time', 150);

/**
 * Configuration options for each flavour.
 *
 */
class Setup extends FormBase {

  /**
   * The site's default country
   * @var string
   */
  private $country;

  /**
   * @var CountryManagerInterface
   */
  private $countryManager;

  /**
   * @var ModuleInstallerInterface
   */
  private $installer;

  /**
   * @var LinkGeneratorInterface
   */
  protected $linkGenerator;

  /**
   * @var EntityTypeManagerInterface
   */
  private $entityTypeManager;
  /**
   * @var \Drupal\Core\Extension\ExtensionList
   */
  private $extensionList;
  /**
   * @var
   */
  private $formBuilder;
  /**
   * @var
   */
  private $defaultLanguage;
  /**
   * @var
   */
  private $moduleHandler;

  /**
   * @var ExtensionDiscovery
   */
  private $extensionDiscovery;


  /**
   *
   * @param ModuleInstallerInterface $module_installer
   * @param LinkGeneratorInterface $link_generator
   * @param EntityTypeManagerInterface $entity_type_manager
   * @param ExtensionList $extension_list
   * @param type $form_builder
   * @param type $language_manager
   * @param ModuleHandler $module_handler
   * @param CountryManagerInterface $country_manager
   */
  public function __construct(ModuleInstallerInterface $module_installer, LinkGeneratorInterface $link_generator, EntityTypeManagerInterface $entity_type_manager, ExtensionList $extension_list, $form_builder, $language_manager, ModuleHandler $module_handler,  CountryManagerInterface $country_manager, ExtensionDiscovery $extension_discovery) {
    $this->installer = $module_installer;
    $this->linkGenerator = $link_generator;
    $this->entityTypeManager = $entity_type_manager;
    $this->extensionList = $extension_list;
    $this->formBuilder = $form_builder;
    $this->defaultLanguage = $language_manager->getDefaultLanguage();
    $this->moduleHandler = $module_handler;
    $this->extensionDiscovery = $extension_discovery;
    $this->countryManager = $country_manager;
    $this->country = $this->config('system.date')->get('country.default');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_installer'),
      $container->get('link_generator'),
      $container->get('entity_type.manager'),
      $container->get('extension.list.module'),
      $container->get('form_builder'),
      $container->get('language_manager'),
      $container->get('module_handler'),
      $container->get('country_manager'),
      new ExtensionDiscovery($container->get('app.root'))
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_setup_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['migrate'] = [
      '#type' => 'submit',
      '#value' => 'Skip setup and migrate from Drupal 7',
      '#weight' => 0,
      '#submit' => ['::migration']
    ];

    $form['features'] = [
      // Hope that yields the current language.
      '#title' => (string) t('Hamlets features'),
      '#type' => 'checkboxes',
      '#options' => [],
      // Everything enabled by default.
      '#default_value' => [],
      '#weight' => 1,
    ];

    foreach ($this->extensionList->getList() as $module_name => $extension) {
      $info = $extension->info;
      if ($info['package'] == 'Cforge features') {
        if (!$this->moduleHandler->moduleExists($module_name)) {
          $form['features']['#options'][$module_name] = $this->t($info['name']);
          $form['features']['#default_value'][] = $module_name;
          $form['features'][$module_name]['#description'] = $this->t($info['description']);
        }
      }
    }

    $form['data_content'] = [
      '#title' => $this->t('Prepare the site with'),
      '#type' => 'radios',
      '#options' => [
        'none' => t('No content'),
        'import' => $this->t('CSV Import Tools'),
      ],
      '#default_value'  => 'none',
      '#weight' => 3,
      '#required' => TRUE,
    ];

    $alt_login_form = $this->formBuilder->getForm('Drupal\alt_login\Settings');
    $form['aliases'] = [
      '#title' => $alt_login_form['aliases']['#title'],
      '#type' => 'select',
      '#options' => $alt_login_form['aliases']['#options'],
      '#default_value' => 'address_name',
      '#weight' => 5
    ];

    $form['default_categories'] = array(
      '#title' => $this->t('Preload ad categories'),
      '#description' => $this->t('Can be modified later'),
      '#type' => 'radios',
      '#options' => [
        0 => $this->t('- None -'),
        'en' => 'English LETS',
        'fr' => 'French',
        'be' => 'SEL Belgique, (2 Tier]'
      ],
      '#default_value' => $this->defaultLanguage->getId(),
      '#weight' => 6,
      '#states' => [
        'visible' => [
          ':input[name=data_content]' => ['value' => 'none'],
        ],
      ],
    );

    $form['site_default_country'] = [
      '#type' => 'select',
      '#title' => $this->t('Country'),
      '#empty_value' => '',
      '#default_value' => $this->country,
      '#options' => $this->countryManager->getList(),
      '#weight' => 7,
      '#required' => TRUE,
      '#disabled' => (bool)$this->country
    ];


    $form['currency_type'] = [
      '#title' => $this->t('Main currency type'),
      '#type' => 'radios',
      '#options' => [
        'hours' => $this->t('Hours'),
        'other' => $this->t('Other'),
      ],
      '#default_value' => 'hours',
      '#required' => TRUE,
      '#weight' => 8,
    ];
    $form['currency_name'] = [
      '#title' => $this->t('Currency name'),
      '#description' => $this->t('Use the plural'),
      '#type' => 'textfield',
      '#placeholder' => $this->t('Units'),
      '#weight' => 9,
      '#states' => [
        'visible' => [
          ':input[name="currency_type"]' => ['value' => 'other'],
        ],
      ],
    ];

    $form['direction'] = [
      '#title' => $this->t('Default transaction direction'),
      '#type' => 'radios',
      '#options' => [
        'credit' => $this->t('Credit (push payment)'),
        'bill' => $this->t('Bill (pull payment)'),
        'both' => $this->t('Both (may be confusing for some users)')
      ],
      '#default_value' => 'credit',
      '#weight' => 10,
    ];

    $form['devel'] = [
      '#title' => t('Devel modules'),
      '#type' => 'checkbox',
      '#weight'=> 15
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => 'Initiate Hamlets',
      '#weight' => 20,
    ];

    return $form;
  }


  public function submitForm(array &$form, FormStateInterface $form_state) {

    $this->installer->install(['default_content']);
    $content_path = DRUPAL_ROOT . '/'.drupal_get_path('module', 'cforge').'/content';
    if (file_exists($content_path)) {
      \Drupal::service('default_content.importer')->importContent('cforge');
    }
    else \Drupal::messenger()->addError($content_path . ' does not exist.');

    $this->setAnonImage('profiles/cforge/themes', 'anonymous.png');

    $available_modules = array_keys($this->extensionDiscovery->scan('module'));
    foreach($available_modules as $module) {
      if (substr($module, 0, 7) == 'cforge_') {
        module_set_weight($module, 101); //This doesn't seem to work anyway
      }
    }

    if ($form_state->getValue('direction') == 'credit') {
      if ($display = EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.bill')) {
        $display->setThirdPartySetting('mcapi_forms', 'permission', 'manage mcapi')->save();
      }
    }
    elseif ($form_state->getValue('direction') == 'bill') {
      if ($display = EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.credit')) {
        $display->setThirdPartySetting('mcapi_forms', 'permission', 'manage mcapi')->save();
      }
    }

    if ($form_state->getValue('default_categories')) {
      $terms = $this->entityTypeManager
        ->getStorage('taxonomy_term')
        ->loadByProperties(['vid' => SMALLAD_CATEGORIES]);
      // Delete any existing terms in the vocab (one is installed by default)
      foreach ($terms as $term) {
        $term->delete();
      }
      $vocab = Vocabulary::load(SMALLAD_CATEGORIES);
      // Now import the terms into the vocab.
      $get_terms = 'vocab_'.$form_state->getValue('default_categories');
      $this->vocabImportRecursive($vocab, $this->$get_terms());
      $vocab->save();
    }

    if (!$this->country) {
      $country_code = $form_state->getValue('site_default_country');
      $field = FieldConfig::load('user.user.address');
      $settings = $field->get('settings');
      $settings['available_countries'][$country_code] = $country_code;
      $field->set('settings', $settings)->save();
      $this->configFactory()->getEditable('system.date')->set('country.default', $country_code)->save();
      $u1 = User::load(1);
      $u1->address->country_code = $country_code;
      $u1->save();
    }

    // Set up the currency by deleting the others.
    if ($c = Currency::load('veur')) {
      $c->delete();
    }
    if ($form_state->getValue('currency_type') == 'hours') {
      if ($c = Currency::load('cc')) {
        $c->delete();
      }
    }
    if ($form_state->getValue('currency_type') == 'cc') {
      if ($c = Currency::load('hhrs')) {
        $c->delete();
      }
    }
    if ($currs = Currency::loadMultiple()) {
      $curr = reset($currs);
      $curr->name = $form_state->getValue('currency_name') ?: 'Units';
      $symbol = strtoupper(substr($curr->name, 0, 1));
      $curr->format = ["<strong>$symbol</strong>", '9999'];
      $curr->save();
    }

    $alt_login_conf = $this->configfactory()->getEditable('alt_login.settings')
      ->set('display_anon', 'Member [user:uid]')
      ->set('display', '[user:address:given_name] [user:address:family_name]')
      ->set('aliases', [$form_state->getvalue('aliases')]);
    if ($form_state->getvalue('aliases') == 'uid') {
      $alt_login_conf
        ->set('display', '[user:address:given_name] [user:address:family_name] ([user:uid])');
    }
    $alt_login_conf->save();

    $modules = array_filter($form_state->getValue('features'));
    switch ($form_state->getValue('data_content')) {
      case 'import':
        $modules[] = 'cforge_import';
        break;
      case 'none':
        cf_neighbourhoods_create(['Centre']);
    }

    if ($form_state->getValue('devel')) {
      $modules = array_merge($modules, [
        'devel_php',
        'dblog',
        'field_ui',
        'views_ui',
        'contextual',
        'admin_toolbar_tools',
        'config'
      ]);
    }

    if ($modules) {
      $this->installer->install($modules);
      $this->installer->uninstall(['default_content']);
    }
    // All imported nodes happen to require our 'public' flag.
    // @todo implement DefaultContentEvents::IMPORT, new ImportEvent($created, $module)
    // @see https://www.drupal.org/docs/creating-custom-modules/subscribe-to-and-dispatch-events
    foreach (Node::loadMultiple() as $node) {
      if ($node->bundle() == 'page') {
        cforge_node_set_anonaccess($node->id(), 1);
      }
    }
    $form_state->setRedirectUrl(
      Url::fromUserInput($this->config('cforge.settings')->get('member_frontpage'))
    );
  }

  /**
   * Submit callback
   */
  public function migration(array &$form, FormStateInterface $form_state) {
    $modules = [
      'migrate',
      'migrate_drupal',
      'migrate_drupal_ui',
      'migrate_plus',
      'migrate_tools',
      'migrate_upgrade',
      'options'
    ];
    $this->installer->install($modules);
    $form_state->setRedirectUrl(Url::fromUserInput('/upgrade'));
  }

  /**
   * Import the terms in a vocabulary or a level of a vocab.
   */
  private function vocabImportRecursive(Vocabulary $vocab, array $termArray, $parent = 0) {
    $w = $hierarchy = 0;
    foreach (Element::Children($termArray) as $name) {
      $props = [
        'name' => $name,
        'weight' => $w++,
        'vid' => $vocab->id(),
        'parent' => $parent,
        'format' => 'plain_text',
      ];
      $term = Term::Create($props);
      $term->save();
      if ($termArray['#children']) {
        // @todo switch over these lines on 8.2.0
        //$vocab->setHierarchy(VocabularyInterface::HIERARCHY_SINGLE);
        $vocab->setHierarchy(TAXONOMY_HIERARCHY_SINGLE);
        $this->vocabImportRecursive($vocab, $termArray[$name], $term->tid);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  private function vocab_en() {
    return [
      '#children' => FALSE,
      'Arts & Culture' => [],
      'Business Services & Clerical' => [],
      'Clothing' => [],
      'Computing & Electronics' => [],
      'Education & Language' => [],
      'Food' => [],
      'Health & Wellness' => [],
      'House & garden' => [],
      'Sports & Leisure' => [],
      'Skills & DIY' => [],
      'Transport' => [],
      'Miscellaneous' => []
    ];
  }

  /**
   * {@inheritdoc}
   */
  private function vocab_fr() {
    return [
      '#children' => FALSE,
      'Administration' => [],
      'Alimentation' => [],
      'Artisanat & Bricolage' => [],
      'Arts & Culture' => [],
      'Cours & Langues' => [],
      'Informatique & Eléctro' => [],
      'Maison & Jardin' => [],
      'Mobilité' => [],
      'Santé & Soins' => [],
      'Sports & Evasion' => [],
      'Vêtements plus' => [],
      'Divers' => [],
    ];
  }
  private function vocab_be() {
    return [
      '#children' => TRUE,
      'A Ménage / Entretien maison' => [
        '#children' => FALSE,
        "Nettoyage/ produits d'entretien" => [],
        'Lessive, repassage' => [],
        'Couture, tricot' => [],
        'Rangement' => [],
        'Ménage - Divers' => [],
      ],
      'B Travaux maisons' => [
        '#children' => FALSE,
        'Travaux lourds' => [],
        'Petits travaux divers' => [],
        'Peinture, tapissage, décoration' => [],
        'Electricité' => [],
        'Plomberie, chauffage' => [],
        'Carrelage, plafonnage, maçonnerie' => [],
        'Menuiserie, planchers, meubles' => [],
        "Isolation, économies d'énergie" => [],
        "Travaux - Divers" => [],
      ],
      'C Enfants / Ados' => [
        '#children' => FALSE,
        'Grossesse et bébé' => [],
        'Baby sitting' => [],
        'Stages, animations créatives' => [],
        'Aide scolaire' => [],
        'Jeux et jouets' => [],
        'Enfants - Divers' => [],
      ],
      'D Alimentation / Gastronomie' => [
        '#children' => FALSE,
        'Nettoyage fruits et légumes' => [],
        'Boissons, soupes, sauces' => [],
        'Plats' => [],
        'Desserts' => [],
        "Tables d'hôtes" => [],
        'Cours, conseils, recettes' => [],
        'Alimentation - Divers' => [],
      ],
      'E Santé / Bien-être / Accompagnement' => [
        '#children' => FALSE,
        'Coiffure, esthétique visage et corps' => [],
        'Massage, thérapies, produits naturels' => [],
        'Personnes âgées, malades, immobilisées' => [],
        "Accompagnement, coaching et conseils 'psy'" => [],
        'Santé - Divers' => [],
      ],
      'F Administration / Gestion' => [
        '#children' => FALSE,
        'Classement de papiers, de dossiers' => [],
        'Dactylographie, mise en page' => [],
        'Courriers, démarches administratives' => [],
        'Assurances, questions juridiques' => [],
        'Fiscalité, pension, chômage' => [],
        'Administration - Divers' => [],
      ],
      'G Cours / Formations / Conseils' => [
        '#children' => FALSE,
        'Langues, conversation, traduction' => [],
        'Ecriture, rédaction, orthographe' => [],
        "Recherche d'emploi" => [],
        'Gestion et animation de groupes' => [],
        'Cours - Divers' => [],
      ],
      'H Jardin / Animaux / Nature' => [
        '#children' => FALSE,
        'Jardin : conception, entretien, conseils' => [],
        'Culture potagère, vergers' => [],
        'Récolte de fruits, légumes, plantes' => [],
        "Taille d'arbres et arbustes" => [],
        'Animaux (domestiques -élevage]' => [],
        'Biodiversité, nichoirs, découverte nature' => [],
        'Engrais, semences, plants, produits divers' => [],
        'Jardin et animaux - Divers' => [],
      ],
      'I Arts / Culture / Sports & loisirs' => [
        '#children' => FALSE,
        'Artisanat, peinture, dessin, sculpture' => [],
        'Musique, chant, danse' => [],
        'Photo, vidéo, cinema' => [],
        'Animations, spectacles' => [],
        'Fêtes : conseils, organisation, aide' => [],
        'Promenades, excursions' => [],
        "Jeux (de société, d'extérieur]" => [],
        'Sports' => [],
        'Culture et sport - Divers' => [],
      ],
      'J Transports / Voyages /Hébergement' => [
        '#children' => FALSE,
        'Courses' => [],
        'Covoiturage et transport' => [],
        'Déménagement' => [],
        'Auto-moto-vélo' => [],
        'Voyages, randonnées' => [],
        'Hébergement, échange maisons' => [],
        'Gardiennage maison' => [],
        'Transport - Divers' => [],
      ],
      'K Informatique /Électroménager' => [
        '#children' => FALSE,
        'Dépannage informatique' => [],
        'Réparation électroménager' => [],
        'Formations Internet / email' => [],
        'Formations logiciels' => [],
        'Informatique, électroménager - Divers' => [],
      ],
      'Z Divers (non classées]' => [
        '#children' => FALSE,
      ]
    ];
  }


  /**
   * Import the anon image.\
   * Uuid is referenced in field.field.user.user.user_picture
   */
  function setAnonImage($dir, $filename) {
    // Where it must go on the host installation.
    $dest = 'public://default_images';
    $props = [
      'uid' => 1,
      'filename' => $filename,
      'uri' => $dir . '/' . $filename,
      'filemime' => 'image/png',
      // 'filesize' => 10279,//@todo get the filesize.
      'status' => TRUE,
    ];
    $file = File::create($props);
    if (file_prepare_directory($dest, FILE_CREATE_DIRECTORY)) {
      $copied = @file_copy($file, $dest . '/' . $filename, FILE_EXISTS_REPLACE);
      if ($copied) {
        $copied->set('uuid', 'ba411b1e-9478-4f42-ab45-69b7857303ed')->save();
      }
    }
  }

}
